<?

 /*************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the PowerDNS JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The PowerDNS JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/pdns/
 *************************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."mutilate.php");

 class Plugin_Backend_pdns extends Base_Plugin_Backend
 {
  function main()
  {
   $this->plugin->l->read("jas");
   $this->plugin->l->read("module_pdns_domains");
   $this->plugin->l->read("module_pdns_records");

   //get main backend component
   if(!($this->component=$this->backend->component("main")))
   {
    $this->e->add("Unable to get main backend component", "error");
    return false;
   }

   if(request("p", 0)=="plugins") //redirect to plugin backend page
    Headers::redirectS($this->component->url("plugin/".$this->plugin->name."/"));

   if(sizeof(request("p"))>=3) //show domain/records
    $this->backend->search("data", "pdns_records", "title", "name");
   else
    $this->backend->search("data", "pdns_domains", "title", "name");


			//search redirect
			if(request("search"))
			{
    if(sizeof(request("p"))>=3) //show domain/records
     $d=Data::cache("pdns_records", "search", "full");
    else
     $d=Data::cache("pdns_domains", "search", "full");

				if($d->fulltextIndex) //full text index exists
					$d->setExtra("index");

				$d->setShowFields("name", "type");
				$d->setValue("show_total", 2);
				$d->setRequestValue("search");

				if($d->get()&&$d->vmax==1) //only 1 entry
    {
     if(sizeof(request("p"))>=3) //show domain/records
      Headers::redirectS($this->component->url("plugin/".$this->plugin->name."/".$d->getValue("type")."/".$d->getValue("name")."/"));
     else
      Headers::redirectS($this->component->url("plugin/".$this->plugin->name."/".$d->getValue("name")."/"));
    }
			}

   return true;
  }

  /*
   * Output
   */

  function output()
  {
   switch(sizeof(request("p")))
   {
    case 4: //type/record.name/
     return $this->recordInfo(request("p", 2), request("p", 3));
    case 3: //domain.name/
     return $this->domainInfo(request("p", 2));
    default:
    {
     if(request("privileges")!==NULL)
      return $this->privileges();

     break;
    }
   }

   return $this->domains();
  }

  function privileges()
  {
   $this->backend->setTitle($this->component->icon("database_dim"),
    $this->component->link($this->plugin->l->t("PDNS"), Array("plugin/".$this->plugin->name."/")),
    $this->component->link($this->plugin->l->t("Database privileges"), Array("plugin/".$this->plugin->name."/?database")));

   //get users used in records
   $users=Array("pdns");

   $d=Data::cache("pdns_records", "view", "all");
   $d->setShowFields("user");
   $d->sortOrder("user");

   $d->clearValues();
   $d->setValue("aggregation_field", "user");

   if($d->get())
    while($d->next())
     if($d->v["user"]&&!in_array($d->v["user"], $users))
      $users[]=$d->v["user"];

   $db=$d->storageObject()->getConfig()["name"];

   //print queries
   $queries="<h2>".$this->plugin->l->t("Views")."</h2>\n";
   $queries.="<div class=\"queries views\">\n";
   $queries.="CREATE OR REPLACE VIEW `".$db."`.`records` AS SELECT `id`, `domain_id`, `type`, `name`, `content`, `ttl`, `prio`, UNIX_TIMESTAMP(`change_date`) change_date, IF(`disabled`=\"true\", 1, 0) disabled, IF(`auth`=\"true\", 1, 0) auth, `ordername` FROM `".$db."`.`pdns_records` WHERE `user` = SUBSTRING_INDEX(USER(), '@', 1) OR `user` = \"\" ORDER BY `name` ASC, `content` ASC;<br/>\n";
   $queries.="CREATE OR REPLACE VIEW `".$db."`.`domains` AS SELECT `id`, `name`, `type`, `master`, UNIX_TIMESTAMP(`last_check`) last_check, `notified_serial`, `account` FROM `".$db."`.`pdns_domains` ORDER BY `name` ASC;<br/>\n";
   $queries.="CREATE OR REPLACE VIEW `".$db."`.`domainmetadata` AS SELECT `id`, `domain_id`, `kind`, `content` FROM `".$db."`.`pdns_domainmetadata` ORDER BY `kind` ASC, `content` ASC;<br/>\n";
   $queries.="</div>\n";

   $queries.="<h2>".$this->plugin->l->t("Privileges")."</h2>\n";
   foreach($users as $user)
   {
    $queries.="<div class=\"queries privileges\">\n";
    $queries.="REVOKE ALL PRIVILEGES ON `".$db."`.`records` FROM '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="REVOKE ALL PRIVILEGES ON `".$db."`.`domains` FROM '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="REVOKE ALL PRIVILEGES ON `".$db."`.`domainmetadata` FROM '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="GRANT SELECT ON `".$db."`.`records` TO '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="GRANT SELECT ON `".$db."`.`domains` TO '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="GRANT SELECT ON `".$db."`.`domainmetadata` TO '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
    $queries.="</div>\n";
   }

   $user="pdns-dyn";
   $queries.="<h2>".$this->plugin->l->t("Dynamic IP privileges")."</h2>\n";
   $queries.="<div class=\"queries privileges\">\n";
   $queries.="REVOKE ALL PRIVILEGES ON `".$db."`.`pdns_records` FROM '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
   $queries.="GRANT SELECT(`id`, `content`, `dynamic`) ON `".$db."`.`pdns_records` TO '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
   $queries.="GRANT UPDATE(`content`) ON `".$db."`.`pdns_records` TO '<div class=\"hilite\">".$user."</div>'@'localhost';<br/>\n";
   $queries.="</div>\n";

   //actions menu
   $this->component->menuItem("actions", $this->plugin->l->t("Domains"), Array("plugin/".$this->plugin->name."/"), $this->plugin->l->t("Show: domains"), "online");


   $output="<div class=\"twothird\">\n";
   $output.=$queries;
   $output.="</div>\n";

   $output.="<div class=\"third\">\n";
   $output.="<h3>".$this->component->icon("execute_dim", "small")." ".$this->plugin->l->t("Actions")."</h3>\n";
   $output.=$this->component->menu("actions");
   $output.="</div>\n";

   return $output;
  }

  function domains()
  {
   //set default title
   $this->backend->setTitle($this->component->icon("online_dim"),
    $this->component->link($this->plugin->l->t("PDNS"), Array("plugin/".$this->plugin->name."/")));

   //get domains
   $d=Data::cache("pdns_domains", "view", "all");
   $d->setShowFields("name", "type", "recordtypes");

   if(request("search"))
   {
    $d->setAction("search");
    $d->setExtra("full");
   }

   if(!$d->get(0, true))
   {
    $this->e->add($d->e);
    return false;
   }

   //get records stats
   $dr=Data::cache("pdns_records", "view", "all");
   $dr->setShowFields("domain_id", "type", "user");
   $dr->sortOrder("domain_id", "type", "user");

   if(!$dr->get())
   {
    $this->e->add($dr->e);
    return false;
   }

   $this->component->tableRow("domains", true);
   $this->component->tableColumn("domains", $this->plugin->l->t("Domain name"), false, false, Array("class" => "name"));
   $this->component->tableColumn("domains", $this->plugin->l->t("Type"), false, false, Array("class" => "type"));
   $this->component->tableColumn("domains", $this->plugin->l->t("Records"), false, false, Array("class" => "records"));
   $this->component->tableColumn("domains", $this->plugin->l->t("Users"), false, false, Array("class" => "users"));
   $this->component->tableColumn("domains", "", false, false, Array("class" => "cms popup right"));

   while($d->next())
   {
    $this->component->tableRow("domains");
    $this->component->tableColumn("domains", $d->v["name"], Array("plugin/".$this->plugin->name."/".$d->v["name"]."/"), false, Array("class" => "name"));
    $this->component->tableColumn("domains", $d->listOptions["type"][$d->v["type"]], false, false, Array("class" => "type"));

    //gather statistics
    $dr->reset();
    $types=Array();
    $users=Array();
    while($dr->search("domain_id", $d->v["id"]))
    {
     if(!isset($types[$dr->v["type"]]))
      $types[$dr->v["type"]]=1;
     else
      $types[$dr->v["type"]]++;

     if($dr->v["user"]&&!in_array($dr->v["user"], $users))
      $users[]=$dr->v["user"];
    }
    sort($users);

    $tmp=Array();
    foreach($types as $type => $count)
     $tmp[]=$this->component->link((in_array($type, $d->v["recordtypes"])?"<b>".$type."</b>":$type), Array("plugin/".$this->plugin->name."/".$d->v["name"]."/", Array("type" => $type)), $dr->listOptions["type"][$type]).($count>1?" (".$count.")":"");
    $types=$tmp;

    $tmp=Array();
    foreach($users as $user)
     $tmp[]=$this->component->link($user, Array("plugin/".$this->plugin->name."/".$d->v["name"]."/", Array("user" => $user)), $user);
    $users=$tmp;

    $this->component->tableColumn("domains", implode(", ", $types), false, false, Array("class" => "records"));
    $this->component->tableColumn("domains", implode(", ", $users), false, false, Array("class" => "users"));

    if($cms=$this->component->cmsOptions($d, false, true))
     $this->component->tableColumn("domains", $this->component->icon("cms").$cms, false, false, Array("class" => "cms popup right"));
    else
     $this->component->tableColumn("domains", "", false, false, Array("class" => "cms"));
   }

   //total
   $this->component->tableRow("domains", false, Array("class" => "total"));

   if(!$d->vmax)
    $this->component->tableColumn("domains", $this->plugin->l->t("No items"), false, false, Array("colspan" => 4));
   else
   {
    $this->component->tableColumn("domains", $this->plugin->l->t("Total"), false, false, Array("colspan" => 3));
    $this->component->tableColumn("domains", $d->vmax." / ".$d->getValue("show_count"), false, false, Array("class" => "nr"));
   }

   if($cms=$this->component->cmsOptions($d, "add", false))
    $this->component->tableColumn("domains", $cms["icon"], $cms["action"], $cms["title"], Array("class" => "cms"));
   else
    $this->component->tableColumn("domains", "", false, false, Array("class" => "cms"));


   //add domain
   $form=$this->component->field($this->plugin->l->t("Name"), $this->component->form->input("text", "name", false, false, false, "placeholder=\"".$this->plugin->l->t("example.com")."\" onchange=\"this.form.submit();\""));
   $form.=$this->component->field(false, $this->component->form->button("", $this->component->icon("online", "small")." ".$this->plugin->l->t("Create"), false, false, "submit"));

   //actions menu
   $this->component->menuItem("actions", $this->plugin->l->t("Database privileges"), Array("plugin/".$this->plugin->name."/?privileges"), $this->plugin->l->t("Show: database privileges"), "database");


   $output="<div class=\"twothird\">\n";
   $output.=$this->component->table("domains");
   $output.="</div>\n";

   $output.="<div class=\"third\">\n";
   $output.="<h3>".$this->component->icon("online_dim", "small")." ".$this->plugin->l->t("New domain")."</h3>\n";
   $output.=$this->component->form($form, $this->component->url("cms/pdns_domains/add/item/"));
   $output.="<h3>".$this->component->icon("execute_dim", "small")." ".$this->plugin->l->t("Actions")."</h3>\n";
   $output.=$this->component->menu("actions");
   $output.="</div>\n";

   return $output;
  }

  function domainInfo($domain_name)
  {
   //get domains
   $d=Data::cache("pdns_domains", "view", "all");
   $d->setShowFields("name", "type", "recordtypes", "subnets");

   $d->setValue("name", $domain_name);

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   $domain_id=$d->v("id");

   //set default title
   $this->backend->setTitle($this->component->icon("online_dim"),
    $this->component->link($this->plugin->l->t("PDNS"), Array("plugin/".$this->plugin->name."/")),
    $this->component->link($d->v("title"), Array("plugin/".$this->plugin->name."/".$d->v("title")."/")));


   //get records stats
   $dr=Data::cache("pdns_records", "view", "all");
   $dr->setShowFields("domain_id", "type", "user");
   $dr->sortOrder("type", "name");
   $dr->setValue("domain_id", $domain_id);

   if(!$dr->get())
   {
    $this->e->add($dr->e);
    return false;
   }

   $this->component->tableRow("domain", true);
   $this->component->tableColumn("domain", $this->plugin->l->t("Domain"), false, false, Array("colspan" => 2));

   $this->component->tableRow("domain", false, Array("hilite"));
   $this->component->tableColumn("domain", $this->plugin->l->t("Name"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", $d->v["name"], Array("plugin/".$this->plugin->name."/".$d->v["name"]."/"), false, Array("class" => "value name text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Type"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", $d->listOptions["type"][$d->v["type"]], false, false, Array("class" => "value type list"));


   //gather statistics
   $types=Array();
   $users=Array();
   while($dr->next())
   {
    if(!isset($types[$dr->v["type"]]))
     $types[$dr->v["type"]]=1;
    else
     $types[$dr->v["type"]]++;

    if($dr->v["user"]&&!in_array($dr->v["user"], $users))
     $users[]=$dr->v["user"];
   }
   sort($users);

   $tmp=Array();
   foreach($types as $type => $count)
    $tmp[]=$this->component->link((in_array($type, $d->v["recordtypes"])?"<b>".$type."</b>":$type), Array(false, Array("type" => $type)), $dr->listOptions["type"][$type]).($count>1?" (".$count.")":"");
   $types=$tmp;

   $tmp=Array();
   foreach($users as $user)
    $tmp[]=$this->component->link($user, Array(false, Array("user" => $user)), $user);
   $users=$tmp;

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Records"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", implode(", ", $types), false, false, Array("class" => "value type text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Users"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", implode(", ", $users), false, false, Array("class" => "value users text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Subnets"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", str_replace(" ", "<br/>", $d->v["subnets"]), false, false, Array("class" => "value subnets text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->component->cmsOptions($d, "edit", true), false, false, Array("colspan" => 2));


   //records
   $dr=Data::cache("pdns_records", "view", "all");
   $dr->setShowFields("type", "user", "name", "content", "dynamic", "related_id", "disabled");
   $dr->sortOrder("type", "name", "content");
   $dr->setValue("domain_id", $domain_id);

   if(request("type")) //type filtering
    $dr->setValue("type", request("type"));
   if(request("user")) //user filtering
    $dr->setValue("user", request("user"));

   if(request("search"))
   {
    $dr->setValue("name", request("search"));
    $dr->setOperator("name", "like");

    unset($dr->field["content"]["value"]);
    $dr->setOperant("name", "OR");

    $dr->setValue("domain_id", $domain_id, 1);

    if(request("type")) //type filtering
     $dr->setValue("type", request("type"), 1);
    if(request("user")) //user filtering
     $dr->setValue("user", request("user"), 1);

    $dr->setValue("content", request("search"), 1);
    $dr->setOperator("content", "like", 1);
   }

   if(!$dr->get(0, true))
   {
    $this->e->add($dr->e);
    return false;
   }

   //resort PTR records
   $from=$to=false;
   while($dr->search("type", "PTR"))
   {
    if($from===false)
     $from=$dr->vc-1;

    $to=$dr->vc;
   }
   $to--;

   if($from!==false&&$to!==false)
    $dr->resort(Array("content", "name"), $from, $to);


   $this->component->tableRow("records", true);
   $this->component->tableColumn("records", $this->plugin->l->t("Type"), false, false, Array("class" => "type"));
   $this->component->tableColumn("records", $this->plugin->l->t("Record"), false, false, Array("class" => "name key"));
   $this->component->tableColumn("records", $this->plugin->l->t("Content"), false, false, Array("class" => "content value"));
   $this->component->tableColumn("records", $this->plugin->l->t("User"), false, false, Array("class" => "user"));
   $this->component->tableColumn("records", "", false, false, Array("class" => "cms popup right"));

   while($dr->next())
   {
    $this->component->tableRow("records", false, Array(($dr->v["related_id"]?"hilite":""), "class" => ($dr->v["disabled"]?"disabled":"")));
    $this->component->tableColumn("records", $dr->v["type"], Array(false, Array("type" => $dr->v["type"])), $dr->listOptions["type"][$dr->v["type"]], Array("class" => "type"));

    if($dr->v["type"]=="PTR") //flip name + content
    {
     $this->component->tableColumn("records", $dr->v["content"], Array("plugin/".$this->plugin->name."/".$dr->v["type"]."/".$dr->v["name"]."/"), false, Array("class" => "content"));
     $this->component->tableColumn("records", $dr->v["name"], false, false, Array("class" => "name"));
    }
    else
    {
     $this->component->tableColumn("records", $dr->v["name"], Array("plugin/".$this->plugin->name."/".$dr->v["type"]."/".$dr->v["name"]."/"), false, Array("class" => "name"));
     $this->component->tableColumn("records", ($dr->v["dynamic"]?"<i>".$dr->v["content"]."</i>":$dr->v["content"]), false, false, Array("class" => "content"));
    }

    $this->component->tableColumn("records", $dr->v["user"], Array(false, Array("user" => $dr->v["user"])), false, Array("class" => "user"));

    if($cms=$this->component->cmsOptions($dr, false, true))
     $this->component->tableColumn("records", $this->component->icon("cms").$cms, false, false, Array("class" => "cms popup right"));
    else
     $this->component->tableColumn("records", "", false, false, Array("class" => "cms"));
   }

   //total
   $this->component->tableRow("records", false, Array("class" => "total"));

   if(!$dr->vmax)
    $this->component->tableColumn("records", $this->plugin->l->t("No items"), false, false, Array("colspan" => 4));
   else
   {
    $this->component->tableColumn("records", $this->plugin->l->t("Total"), false, false, Array("colspan" => 3));
    $this->component->tableColumn("records", $dr->vmax." / ".$dr->getValue("show_count"), false, false, Array("class" => "nr"));
   }

   if($cms=$this->component->cmsOptions($dr, "add", false))
    $this->component->tableColumn("records", $cms["icon"], $cms["action"]."?domain_id=".$domain_id, $cms["title"], Array("class" => "cms"));
   else
    $this->component->tableColumn("records", "", false, false, Array("class" => "cms"));


   //add record
   $form=$this->component->field($this->plugin->l->t("Type"), $this->component->form->select(false, "type", $dr->listOptions["type"], request("type"), false, false, "onchange=\"this.form.submit();\""));
   $form.=$this->component->form->input("hidden", "domain_id", $domain_id);
   $form.=$this->component->field(false, $this->component->form->button("", $this->component->icon("online", "small")." ".$this->plugin->l->t("Create"), false, false, "submit"));


   $output="<div class=\"twothird records\">\n";
   $output.=$this->component->table("records");
   $output.="</div>\n";

   $output.="<div class=\"third\">\n";
   $output.="<h3>".$this->component->icon("online_dim", "small")." ".$this->plugin->l->t("New domain record")."</h3>\n";
   $output.=$this->component->form($form, $this->component->url("cms/pdns_records/add/item/"))."<br/>";
   $output.=$this->component->table("domain");
   $output.="</div>\n";

   return $output;
  }

  function recordInfo($record_type, $record_name)
  {
   //get domains
   $dr=Data::cache("pdns_records", "view", "all");

   $dr->setValue("type", $record_type);
   $dr->setValue("name", $record_name);

   if(!$dr->get(1)||!$dr->next())
   {
    $this->e->add($dr->e);
    return false;
   }

   $domain_id=$dr->v["domain_id"]["id"];

   //get domains
   $d=Data::cache("pdns_domains", "view", "all");
   $d->setShowFields("name", "type", "recordtypes", "subnets");

   $d->setValue("id", $domain_id);

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   //set default title
   $this->backend->setTitle($this->component->icon("online_dim"),
    $this->component->link($this->plugin->l->t("PDNS"), Array("plugin/".$this->plugin->name."/")),
    $this->component->link($d->v("title"), Array("plugin/".$this->plugin->name."/".$d->v("title")."/")),
    $this->component->link($dr->v["type"], Array("plugin/".$this->plugin->name."/".$d->v("title")."/", Array("type" => $dr->v["type"]))),
    $this->component->link($dr->v["name"], Array("plugin/".$this->plugin->name."/".$dr->v["type"]."/".$dr->v["name"]."/")));


   //render info
   $dr->reset();
   $recordInfo="";
   while($dr->next())
   {
    if($cms=$this->component->cmsOptions($dr, false, true))
     $recordInfo.="<div class=\"cms popup right\" style=\"float: right; margin-bottom: -20px;\">".$this->component->icon("cms", "small").$cms."</div>\n";

    $recordInfo.=$this->component->viewItem($dr)."<br/>\n";
   }

   //replace reference links with plugin/pdns/ links
   $recordInfo=preg_replace("!(href=\"[^\"]*)data/pdns_domains/[^\"]+([^>]+>([^<]+)\s)!", "\\1plugin/".$this->plugin->name."/\\3/\\2", $recordInfo);
   $recordInfo=preg_replace("!(href=\"[^\"]*)data/pdns_records/[^\"]+([^>]+>([A-Z]+) ([^<]+)\s)!", "\\1plugin/".$this->plugin->name."/\\3/\\4/\\2", $recordInfo);


   //get records stats
   $dr=Data::cache("pdns_records", "view", "all");
   $dr->setShowFields("domain_id", "type", "user");
   $dr->sortOrder("type", "name");
   $dr->setValue("domain_id", $domain_id);

   if(!$dr->get())
   {
    $this->e->add($dr->e);
    return false;
   }

   $this->component->tableRow("domain", true);
   $this->component->tableColumn("domain", $this->plugin->l->t("Domain"), false, false, Array("colspan" => 2));

   $this->component->tableRow("domain", false, Array("hilite"));
   $this->component->tableColumn("domain", $this->plugin->l->t("Name"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", $d->v["name"], Array("plugin/".$this->plugin->name."/".$d->v["name"]."/"), false, Array("class" => "value name text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Type"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", $d->listOptions["type"][$d->v["type"]], false, false, Array("class" => "value type list"));

   //gather statistics
   $types=Array();
   $users=Array();
   while($dr->next())
   {
    if(!isset($types[$dr->v["type"]]))
     $types[$dr->v["type"]]=1;
    else
     $types[$dr->v["type"]]++;

    if($dr->v["user"]&&!in_array($dr->v["user"], $users))
     $users[]=$dr->v["user"];
   }
   sort($users);

   $tmp=Array();
   foreach($types as $type => $count)
    $tmp[]=$this->component->link((in_array($type, $d->v["recordtypes"])?"<b>".$type."</b>":$type), Array("plugin/".$this->plugin->name."/".$d->v["name"]."/", Array("type" => $type)), $dr->listOptions["type"][$type]).($count>1?" (".$count.")":"");
   $types=$tmp;

   $tmp=Array();
   foreach($users as $user)
    $tmp[]=$this->component->link($user, Array("plugin/".$this->plugin->name."/".$d->v["name"]."/", Array("user" => $user)), $user);
   $users=$tmp;


   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Records"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", implode(", ", $types), false, false, Array("class" => "value type text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Users"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", implode(", ", $users), false, false, Array("class" => "value users text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->plugin->l->t("Subnets"), false, false, Array("class" => "key"));
   $this->component->tableColumn("domain", str_replace(" ", "<br/>", $d->v["subnets"]), false, false, Array("class" => "value subnets text"));

   $this->component->tableRow("domain");
   $this->component->tableColumn("domain", $this->component->cmsOptions($d, "edit", true), false, false, Array("colspan" => 2));


   //add record
   $form=$this->component->field($this->plugin->l->t("Type"), $this->component->form->select(false, "type", $dr->listOptions["type"], $record_type, false, false, "onchange=\"this.form.submit();\""));
   $form.=$this->component->form->input("hidden", "domain_id", $domain_id);
   $form.=$this->component->field(false, $this->component->form->button("", $this->component->icon("online", "small")." ".$this->plugin->l->t("Create"), false, false, "submit"));


   $output="<div class=\"twothird records\">\n";
   $output.=$recordInfo;
   $output.="</div>\n";

   $output.="<div class=\"third\">\n";
   $output.="<h3>".$this->component->icon("online_dim", "small")." ".$this->plugin->l->t("New domain record")."</h3>\n";
   $output.=$this->component->form($form, $this->component->url("cms/pdns_records/add/item/"))."<br/>";
   $output.=$this->component->table("domain");

   $output.="</div>\n";

   return $output;
  }

 }
?>
