<?

 /*************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the PowerDNS JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The PowerDNS JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/pdns/
 *************************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["project"]["plugins"]."pdns/modules/records.php");

 class Plugin_Module_pdns_domains extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->data->title="PowerDNS Domains";

   $this->data->addField("id", "nr", "all", "identifier", 1, false, 0, "ID");

   $this->data->addField("name", "hostname", "all", "title;trim", 3, 255, false, "Name", "example.com");

   $this->data->addField("type", "list", "all", "", 1, 6, "NATIVE", "Type");
   $this->data->listOptions("type", Array("NATIVE" => "Native", "MASTER" => "Master", "SLAVE" => "Slave"));


   //manage
   $this->data->addField("recordtypes", "multilist", "all", "", false, 100, Array("SOA", "NS", "MX", "A", "AAAA"), "Record types");
   $this->data->listOptions("recordtypes", Array("A" => "Address (A)", "AAAA" => "Address IPV6 (AAAA)", "MX" => "Mail exchange (MX)", "NS" => "Name server (NS)", "SOA" => "Start Of Authority (SOA)"));

   $this->data->addField("subnets", "text", "all", "", false, 255, "", "Subnets", "192.168.0 2001:0db8 etc.");


   //Meta
   $this->data->addField("master", "text", "sql", "", 0, 128, false, "Master");
   $this->data->addField("last_check", "datetime", "show;sql", "", 0, false, "now", "Last checked");
   $this->data->addField("notified_serial", "nr", "sql", "", 0, false, 0, "Notified serial");
   $this->data->addField("account", "text", "sql", "", 0, 40, "", "Account");

   $this->data->addReference("records", "pdns_records", "domain_id", "1:*", "id");

   if(isset($this->data->field["records"]))
   {
    $this->data->field["records"]["min"]=0;
    $this->data->field["records"]["use"]="show";
   }


   $this->data->addUnique("name_index", "name");

   $this->data->group("Manage", "recordtypes", "subnets");
   $this->data->group("Meta", "master", "last_check", "notified_serial", "account");
   $this->data->sortOrder("name");
  }


  function addPreSubmitErrorCheck(){return $this->customizeFieldsPre();}
  function copyPreSubmitErrorCheck(){return $this->customizeFieldsPre();}
  function editPreSubmitErrorCheck(){return $this->customizeFieldsPre();}

  function customizeFieldsPre()
  {
   if(($name=strtolower($this->data->getValue("name")))&&$name!=$this->data->getValue("name")) //case changed
    $this->data->setValue("name", $name);

   return true;
  }


  function addPostSubmit(){return $this->customizeFieldsPost();}
  function copyPostSubmit(){return $this->customizeFieldsPost();}
  function editPostSubmit(){return $this->customizeFieldsPost();}

  function customizeFieldsPost()
  {
   //manage records
   if(($recordtypes=$this->data->getValue("recordtypes"))&&($domain=$this->data->getValue("id"))&&($name=$this->data->getValue("name")))
   {
    //get records for this domain
    $d=Data::cache("pdns_records", "view", "all");
    $d->setShowFields("type", "name", "content");
    $d->sortOrder("type", "name", "content");

    $d->clearValues();
    $d->setValue("domain_id", $domain, 0);

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    if(!$d->vmax)
    {
     $d->clearValues();
     $d->e->clear();
    }

    //find first domain NS record
    $ns=false;
    $d->reset();
    while($d->search("type", "NS"))
    {
     if($d->v["name"]==$name) //matched domain name
     {
      $ns=$d->v["content"];
      break;
     }
    }

    if(!$ns) //no record found
     $ns="ns".($this->data->getValue("type")=="MASTER"?1:($this->data->getValue("type")=="SLAVE"?2:"")).".".$name;

    //get subnets
    if($subnets=$this->data->getValue("subnets"))
    {
     $tmp=Array();
     foreach(explode(" ", $subnets) as $subnet)
     {
      if(!($arpa=Plugin_Module_pdns_records::IP2ARPA($subnet)))
      {
       $this->data->e->add("Incorrect subnet: \"".$subnet."\"", "error");
       return false;
      }

      $tmp[]=$arpa;
     }

     $subnets=$tmp;
    }
    else
     $subnets=Array();

    //define records to manage
    $manage=Array(
     "NS" => Array(), //ensure NS is checked before SOA (latter needs NS record)
     "SOA" => Array(),
     "MX" => Array(),
     "A" => Array(),
     "AAAA" => Array(),
    );


    if(in_array("SOA", $recordtypes))
    {
     $manage["SOA"]["main"]=Array(
      "name" => $name,
      "content" => $ns." " //primary name server
       ."hostmaster.".$name." " //maintainer email
       .mktime()." " //set serial (must be incremental on every change: unix time!)
       .$d->getDefaultValue("ttl")." " //refresh
       .$d->getDefaultValue("ttl")." " //retry
       .(7*86400)." " //expire
       .$d->getDefaultValue("ttl") #minimum
      ,
     );

     foreach($subnets as $subnet) //add SOA records for all subnets
     {
      $manage["SOA"][$subnet]=Array(
       "name" => $subnet,
       "content" => $ns." " //primary name server
        ."hostmaster.".$name." " //maintainer email
        .mktime()." " //set serial (must be incremental on every change: unix time!)
        .$d->getDefaultValue("ttl")." " //refresh
        .$d->getDefaultValue("ttl")." " //retry
        .(7*86400)." " //expire
        .$d->getDefaultValue("ttl") #minimum
       ,
      );
     }
    }


    if(in_array("NS", $recordtypes))
    {
     $manage["NS"]["main"]=Array(
      "name" => $name,
      "content" => $ns,
     );

     $d->reset();
     foreach($subnets as $subnet) //add NS records for all subnets for each domain NS record
     {
      while($d->search("type", "NS"))
      {
       $manage["NS"][$subnet." ".$d->v["content"]]=Array(
        "name" => $subnet,
        "content" => $d->v["content"],
       );
      }

      if(!isset($manage["NS"][$subnet." ".$ns])) //primary name server not yet in list (no domain NS records in DB yet)
      {
       $manage["NS"][$subnet." ".$ns]=Array(
        "name" => $subnet,
        "content" => $ns,
       );
      }
     }
    }


    if(in_array("MX", $recordtypes))
    {
     $manage["MX"]["main"]=Array(
      "name" => $name,
      "content" => $name,
     );
    }


    if(in_array("A", $recordtypes))
    {
     $manage["A"]["domain"]=Array(
      "name" => $name,
      "content" => "",
      "dynamic" => true,
     );

     //NS A records
     $d->reset();
     while($d->search("type", "NS")) //add A records for domains NS records
     {
      if($d->v["name"]!=$name) //not a domains record
       continue;

      $manage["A"][$d->v["content"]]=Array(
       "name" => $d->v["content"],
       "content" => "",
       "dynamic" => true,
      );
     }

     if(!isset($manage["A"][$ns])) //primary name server not yet in list (no domain NS records in DB yet)
     {
      $manage["A"][$ns]=Array(
       "name" => $ns,
       "content" => "",
       "dynamic" => true,
      );
     }

     //MX A records
     if(in_array("A", $recordtypes))
     {
      $manage["A"]["mx"]=Array(
       "name" => $manage["MX"]["main"]["content"],
       "content" => "",
       "dynamic" => true,
      );
     }
    }


    if(in_array("AAAA", $recordtypes))
    {
     $manage["AAAA"]["domain"]=Array(
      "name" => $name,
      "content" => "",
      "dynamic" => true,
     );

     //NS AAAA records
     $d->reset();
     while($d->search("type", "NS")) //add AAAA records for domains NS records
     {
      if($d->v["name"]!=$name) //not a domains record
       continue;

      $manage["AAAA"][$d->v["content"]]=Array(
       "name" => $d->v["content"],
       "content" => "",
       "dynamic" => true,
      );
     }

     if(!isset($manage["AAAA"][$ns])) //primary name server not yet in list (no domain NS records in DB yet)
     {
      $manage["AAAA"][$ns]=Array(
       "name" => $ns,
       "content" => "",
       "dynamic" => true,
      );
     }

     //MX AAAA records
     if(in_array("AAAA", $recordtypes))
     {
      $manage["AAAA"]["mx"]=Array(
       "name" => $manage["MX"]["main"]["content"],
       "content" => "",
       "dynamic" => true,
      );
     }
    }


    //add/update records based on what to manage
    $d2=Data::cache("pdns_records", "add", "item");
    foreach(array_keys($manage) as $type)
    {
     foreach($manage[$type] as $key => $record)
     {
      //record type specific
      $action=false; //don't add/edit unless overwritten below
      switch($type)
      {
       case "SOA":
       {
        $d->reset();
        $action="add"; //add record (unless found)
        while($d->search("type", $type)) //find domain SOA record
        {
         if($d->v["name"]==$record["name"])
         {
          $action=$d->v["id"]; //update record
          break;
         }
        }

        break;
       }
       case "NS":
       {
        $d->reset();
        $action="add"; //add record (unless found)
        while($d->search("type", $type)) //find record found
        {
         if($d->v["name"]==$record["name"]&&$d->v["content"]==$record["content"])
         {
          $action=false; //don't update
          break;
         }
        }

        break;
       }
       case "MX":
       {
        $d->reset();
        if($d->search("type", $type)) //record found
        {
         $action=false; //don't update record
         $manage[$type][$key]["content"]=$d->v["content"]; //use existing MX record

         if(isset($manage["A"]["mx"])) //MX record managed
          $manage["A"]["mx"]["name"]=$d->v["content"]; //update existing A record name for MX entry
         if(isset($manage["AAAA"]["mx"])) //MX record managed
          $manage["AAAA"]["mx"]["name"]=$d->v["content"]; //update existing AAAA record name for MX entry
        }
        else //not found
         $action="add"; //add record

        break;
       }
       default: //A/AAAA
       {
        $d->reset();
        $action="add"; //add record (unless found)
        while($d->search("type", $type)) //find record found
         if($d->v["name"]==$record["name"])
          $action=false; //don't update

        break;
       }
      }

      //determine add/edit
      if(!$action) //don't add/edit record
       continue;
      else if($action==="add")
       $d2->setAction("add");
      else //edit
       $d2->setAction("edit");

      $d2->e->clear();
      $d2->clearValues();
      $d2->setValue("domain_id", $domain);
      $d2->setValue("type", $type);

      if($record["name"]==$name)
       $d2->setValue("name", ""); //force domain name as hostname
      else
       $d2->setValue("name", $record["name"]);

      $d2->setValue("content", $record["content"]);

      if(isset($record["dynamic"]))
       $d2->setValue("dynamic", $record["dynamic"]);
      else
       $d2->setValue("dynamic", false);

      if($action!=="add") //set record id for updating
       $d2->setValue("id", $action);

      if(!$d2->submit())
      {
       $this->data->e->add($d2->e);
       return false;
      }
      else
      {
       if($action=="add") //add to search list
       {
        $record["id"]=$d2->getValue("id");
        $record["domain_id"]=$domain;
        $record["type"]=$type;

        $c=$d->vmax;
        foreach($record as $k => $v)
         $d->setValue($k, $v, $c);
       }

       $this->data->e->add($type." record ".($action==="add"?"added":"updated"), "note", $record["name"].($record["content"]?": ".$record["content"]:""));
      }
     }
    }
   }

   return true;
  }
 }
?>
