<?

 /*************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the PowerDNS JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The PowerDNS JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/pdns/
 *************************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["include"]."mutilate.php");

 class Plugin_Module_pdns_records extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->data->title="PowerDNS Records";

   $this->data->addField("id", "nr", "all", "identifier", 1, false, 0, "ID");

   $this->data->addField("domain_id", "nr", "all", "", 1, false, 0, "Domain");
   $this->data->addReference("domain_id", "pdns_domains", "id", "*:1");

   $this->data->addField("type", "text", "all", "reload;title", 1, 10, "A", "Type");
   $this->data->listOptions("type", Array("" => "", "A" => "Address (A)", "AAAA" => "Address IPV6 (AAAA)", "CNAME" => "Canonical name (CNAME)", "MX" => "Mail exchange (MX)", "NS" => "Name server (NS)", "PTR" => "Pointer (PTR)", "SOA" => "Start Of Authority (SOA)"));

   $this->data->addField("name", "hostname", "all", "title;trim", 0, 255, false, "Name", "www");

   $this->data->addField("content", "text", "all", "trim", 0, 64000, "", "Content");
   $this->data->addField("dynamic", "text", "all", "reload", 0, 255, false, "Dynamic IP key", "Dynamic IP update key, leave empty for static IP");

   $this->data->addField("ptr", "boolean", "", "reload", false, false, false, "Pointer record");
   $this->data->addField("related_id", "nr", "sql;show", "", 0, false, 0, "Related record");
   $this->data->addReference("related_id", "pdns_records", "id", "*:0");

   $this->data->addField("soaname", "hostname", "", "trim", 3, 255, false, "Name server");
   $this->data->addField("soaemail", "hostname", "", "trim", 3, 255, false, "Email address");
   $this->data->addField("soaserial", "nr", "", "disabled", 1, false, false, "Domain serial");

   $this->data->addField("soarefresh", "nr", "", "", 1, false, 3600, "Refresh");
   $this->data->addField("soaretry", "nr", "", "", 1, false, 3600, "Retry");
   $this->data->addField("soaexpire", "nr", "", "", 1, false, 86400*7, "Expire");

   $this->data->addField("ttl", "nr", "all", "", 0, false, 3600, "Time To Live");
   $this->data->listOptions("ttl", Array("" => "", "300" => "5 minutes", "1800" => "30 minutes", "3600" => "1 hour", "86400" => "1 day", 86400*7 => "1 week"));

   $this->data->listOptions("soarefresh", $this->data->listOptions["ttl"]);
   $this->data->listOptions("soaretry", $this->data->listOptions["ttl"]);
   $this->data->listOptions("soaexpire", $this->data->listOptions["ttl"]);

   $this->data->addField("prio", "nr", "sql", "dropdown", 0, 255, false, "Priority");

   $this->data->addField("change_date", "datetime", "show;sql", "forceUpdate", 0, false, "now", "Last updated");
   $this->data->addField("auth", "boolean", "all", "", false, false, true, "Authoritive");
   $this->data->addField("disabled", "boolean", "all", "", false, false, false, "Disabled");

   $this->data->addField("user", "text", "all", "", false, 25, "", "MySQL username", "Leave empty to allow all users to see this record");
   $this->data->listOptions("user", Array("", "pdns"));

   $this->data->addField("ordername", "text", "sql", "", false, 255, false, "Order name");


   $this->data->addIndex("nametype_index", "name", "type");
   $this->data->addIndex("domain_id", "domain_id");
   $this->data->addIndex("recordorder", "domain_id", "ordername");

   $this->data->group("Meta", "ttl", "soarefresh", "soaretry", "soaexpire", "prio", "change_date", "auth", "disabled", "user", "ordername");
   $this->data->sortOrder("domain_id", "type", "name");
  }

  function postInit()
  {
   if($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit")
   {
    if(!$this->customizeFields())
     return false;

    if(!$this->getUserInfo())
     return false;
   }

   //disallow domain to be changed
   if($this->data->action=="edit")
    $this->data->field["domain_id"]["properties"].=";disabled";

   return true;
  }

  function viewPreGetErrorCheck()
  {
   //convert PTR names to IP
   if($this->data->getValue("type")=="PTR")
    if(preg_match(REGEXP_DATA_IP, $this->data->getValue("name"))) //IPV4/IPV6 address
     if($arpa=$this->IP2ARPA($this->data->getValue("name"))) //try to convert IP address to ARPA address
      $this->data->setValue("name", $arpa);

   return true;
  }

  function viewPostGet(){return $this->customizeFields();}
  function searchPostGet(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   //remove domain name from record name
   if(($domain=$this->data->getValue("domain_id"))&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
   {
    $d=Data::cache("pdns_domains", "view", "item");
    $d->setShowFields("name", "subnets");
    $d->setValue("id", $domain);

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    if(!$d->next())
    {
     $this->data->e->add("Unknown domain", "error");
     return false;
    }

    if(($name=$this->data->getValue("name"))&&strstr($name, $d->v["name"]))
    {
     $name=str_replace($d->v["name"], "", $name);

     if($name) //remove last dot
      $name=substr($name, 0, -1);

     $this->data->setValue("name", $name);
    }
   }

   //modify based on record type
   switch($this->data->getValue("type"))
   {
    case "A":
    {
     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("name", $this->getRecords($domain));

     $this->data->field["content"]["title"]="IPv4 address";
     $this->data->field["content"]["type"]="regexp";
     $this->data->regexps("content", REGEXP_DATA_IPV4);

     if(isset($d)) //domain gathered
     {
      $content=Array("" => ""); //default options
      foreach(explode(" ", $d->v["subnets"]) as $subnet)
       if(preg_match("/^[0-9\.]+$/", $subnet)) //ipv4
        $content[$subnet."."]=$subnet." (subnet)";

      if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
       $content=array_merge($content, (sizeof($content)>1?Array(" " => ""):Array()), $this->getIPs($domain, 4));

      $this->data->listOptions("content", $content);
     }

     $this->data->resetField("dynamic");
     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("dynamic", $this->getDynamicIPKeys($domain));

     if($this->data->getValue("dynamic"))
     {
      $this->data->field["content"]["min"]=0;
      $this->data->resetField("ptr");
     }
     else
     {
      $this->data->field["content"]["min"]=1;
      $this->data->field["ptr"]["use"]="form;show";
     }

     $this->data->resetField("prio");
     break;
    }
    case "AAAA":
    {
     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("name", $this->getRecords($domain));

     $this->data->field["content"]["title"]="IPv6 address";
     $this->data->field["content"]["type"]="regexp";
     $this->data->regexps("content", REGEXP_DATA_IPV6);

     if(isset($d)) //domain gathered
     {
      $content=Array("" => ""); //default options
      foreach(explode(" ", $d->v["subnets"]) as $subnet)
       if(preg_match("/^[0-F\:]+$/i", $subnet)) //ipv6
        $content[$subnet.":"]=$subnet." (subnet)";

      if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
       $content=array_merge($content, (sizeof($content)>1?Array(" " => ""):Array()), $this->getIPs($domain, 6));

      $this->data->listOptions("content", $content);
     }

     $this->data->resetField("dynamic");
     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("dynamic", $this->getDynamicIPKeys($domain));

     if($this->data->getValue("dynamic"))
     {
      $this->data->field["content"]["min"]=0;
      $this->data->resetField("ptr");
     }
     else
     {
      $this->data->field["content"]["min"]=1;

      $this->data->field["ptr"]["use"]="form;show";
      $this->data->setRequestValue("ptr");

      if($this->data->getValue("ptr"))
       $this->data->field["dynamic"]["properties"]=";hidden";
      else
       $this->data->resetField("dynamic");
     }

     $this->data->resetField("prio");
     break;
    }
    case "CNAME":
    {
     $this->data->field["name"]["min"]=3;

     $this->data->field["content"]["title"]="Domain name";
     $this->data->field["content"]["type"]="hostname";
     $this->data->field["content"]["min"]=3;

     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("content", $this->getRecords($domain));

     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->resetField("prio");
     break;
    }
    case "MX":
    {
     $this->data->resetField("name");
     $this->data->field["name"]["use"]="sql;show";

     $this->data->field["content"]["title"]="Domain name";
     $this->data->field["content"]["type"]="hostname";
     $this->data->field["content"]["min"]=3;

     if(isset($d)) //domain gathered
     {
      $content=Array( //default options
       "" => "",
       "mail.".$d->v["name"] => "mail.".$d->v["name"],
       "mx.".$d->v["name"] => "mx.".$d->v["name"],
      );

      if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
       $content=array_merge($content, Array(" " => ""), $this->getRecords($domain));

      $this->data->listOptions("content", $content);
     }

     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->field["prio"]["use"].=";form;show";
     break;
    }
    case "NS":
    {
     $this->data->field["content"]["title"]="Domain name";
     $this->data->field["content"]["type"]="hostname";
     $this->data->field["content"]["min"]=3;

     if(isset($d)) //domain gathered
     {
      $content=Array( //default options
       "" => "",
       "ns.".$d->v["name"] => "ns.".$d->v["name"],
       "ns1.".$d->v["name"] => "ns1.".$d->v["name"],
       "ns2.".$d->v["name"] => "ns2.".$d->v["name"],
       " " => "",
      );

      if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
       $content=array_merge($content, Array(" " => ""), $this->getRecords($domain));

      $this->data->listOptions("content", $content);
     }

     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->resetField("prio");
     break;
    }
    case "PTR":
    {
     $this->data->field["name"]["title"]="IP / ARPA";
     $this->data->field["name"]["text"]="192.168.0 / 0.168.192.in-addr.arpa<br/>192.168.0.1 / 1.0.168.192.in-addr.arpa<br/>2001:0db8: / 8.b.d.0.1.0.0.2.ip6.arpa<br/>2001:0db8:0000:0000:0000:ff00:0042:8329 / 9.2.3.8.2.4.0.0.0.0.f.f.0.0.0.0.0.0.0.0.0.0.0.0.8.b.d.0.1.0.0.2.ip6.arpa";
     $this->data->field["name"]["type"]="hostname";
     $this->data->field["name"]["min"]=3;

     if(isset($d)) //domain gathered
     {
      $content=Array("" => ""); //default options
      foreach(explode(" ", $d->v["subnets"]) as $subnet)
      {
       if(preg_match("/^[0-9\.]+$/", $subnet)) //ipv4
        $content[$subnet."."]=$subnet." (subnet)";
       else if(preg_match("/^[0-F\:]+$/i", $subnet)) //ipv6
        $content[$subnet.":"]=$subnet." (subnet)";
      }

      if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
       $content=array_merge($content, (sizeof($content)>1?Array(" " => ""):Array()), $this->getIPs($domain));

      $this->data->listOptions("name", $content);
     }

     $this->data->field["content"]["title"]="Domain name";
     $this->data->field["content"]["type"]="hostname";
     $this->data->field["content"]["min"]=3;

     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("content", $this->getRecords($domain));

     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->resetField("prio");
     break;
    }
    case "SOA":
    {
     $this->data->resetField("content");
     $this->data->field["content"]["use"]="sql;show";

     if($domain&&($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit"))
      $this->data->listOptions("soaname", $this->getRecords($domain));

     if(isset($d))
     {
      $email=Array(
       "",
       "dns.".$d->v["name"],
       "hostmaster.".$d->v["name"],
       "webmaster.".$d->v["name"],
      );

      $this->data->listOptions("soaemail", $email);
     }

     $this->data->field["soaname"]["use"].="form;show";
     $this->data->field["soaemail"]["use"].="form;show";
     $this->data->field["soaserial"]["use"].="form;show";
     $this->data->field["soarefresh"]["use"].="form;show";
     $this->data->field["soaretry"]["use"].="form;show";
     $this->data->field["soaexpire"]["use"].="form;show";

     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->resetField("prio");
     break;
    }
    default:
    {
     $this->data->resetField("type");
     $this->data->resetField("name");
     $this->data->resetField("content");
     $this->data->field["dynamic"]["properties"]=";hidden";
     $this->data->resetField("ptr");
     $this->data->resetField("prio");
     break;
    }
   }

   //modify values based on record type
   for($c=0;$c<$this->data->vmax;$c++)
   {
    switch($this->data->getValue("type", $c))
    {
     case "A":
     case "AAAA":
     {
      $this->data->setRequestValue("ptr", $c);

      if($this->data->action=="add")
      {
       if(isset($d)&&$d->v["subnets"]&&!$this->data->check("valuetype", "request", "ptr")) //subnets enabled for domain and no value submitted
        $this->data->setValue("ptr", true);
      }
      else if($this->data->action=="copy"||$this->data->action=="edit")
      {
       if(!$this->data->check("valuetype", "request", "ptr")) //no value submitted
       {
        //get value based on related record
        if($this->data->getValue("related_id", $c))
         $this->data->setValue("ptr", true, $c);
        else
         $this->data->setValue("ptr", false, $c);
       }
      }

      break;
     }
     case "PTR":
     {
      if($ip=$this->ARPA2IP($this->data->getValue("name", $c))) //try to convert arpa address to IP address
       $this->data->setValue("name", $ip, $c);

      break;
     }
     case "SOA":
     {
      $this->data->setRequestValue("soaname", $c);
      $this->data->setRequestValue("soaemail", $c);
      $this->data->setRequestValue("soaserial", $c);
      $this->data->setRequestValue("soarefresh", $c);
      $this->data->setRequestValue("soaretry", $c);
      $this->data->setRequestValue("soaexpire", $c);

      if($content=$this->data->getValue("content", $c))
      {
       $content=explode(" ", $content);

       if(!$this->data->getValue("soaname", $c))
        $this->data->setValue("soaname", $content[0], $c);
       if(!$this->data->getValue("soaemail", $c))
        $this->data->setValue("soaemail", $content[1], $c);
       if(!$this->data->getValue("soaserial", $c))
        $this->data->setValue("soaserial", $content[2], $c);
       if(!$this->data->getValue("soarefresh", $c))
        $this->data->setValue("soarefresh", $content[3], $c);
       if(!$this->data->getValue("soaretry", $c))
        $this->data->setValue("soaretry", $content[4], $c);
       if(!$this->data->getValue("soaexpire", $c))
        $this->data->setValue("soaexpire", $content[5], $c);
      }

      break;
     }
     default: break;
    }
   }

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->customizePreSubmitErrorCheck();}
  function copyPreSubmitErrorCheck(){return $this->customizePreSubmitErrorCheck();}
  function editPreSubmitErrorCheck(){return $this->customizePreSubmitErrorCheck();}

  function customizePreSubmitErrorCheck()
  {
   //convert values for db storage
   switch($this->data->getValue("type"))
   {
    case "NS":
    {
     //convert IP to ARPA
     if(preg_match(REGEXP_DATA_IP, $this->data->getValue("name"))) //IPV4/IPV6 address
      if($arpa=$this->IP2ARPA($this->data->getValue("name"))) //try to convert IP address to ARPA address
       $this->data->setValue("name", $arpa);

     break;
    }
    case "PTR":
    {
     //convert IP to ARPA
     if(preg_match(REGEXP_DATA_IP, $this->data->getValue("name"))) //IPV4/IPV6 address
      if($arpa=$this->IP2ARPA($this->data->getValue("name"))) //try to convert IP address to ARPA address
       $this->data->setValue("name", $arpa);

     break;
    }
    case "SOA":
    {
     $this->data->setValue("soaserial", mktime()); //overwrite serial (must be incremental on every change: unix time!)

     //convert field values to content
     if($content=$this->data->getValue("content"))
     {
      $content=explode(" ", $content);

      if($this->data->getValue("soaname"))
       $content[0]=$this->data->getValue("soaname");
      if($this->data->getValue("soaemail"))
       $content[1]=str_replace("@", ".", $this->data->getValue("soaemail"));
      if($this->data->getValue("soaserial"))
       $content[2]=$this->data->getValue("soaserial");
      if($this->data->getValue("soarefresh"))
       $content[3]=$this->data->getValue("soarefresh");
      if($this->data->getValue("soaretry"))
       $content[4]=$this->data->getValue("soaretry");
      if($this->data->getValue("soaexpire"))
       $content[5]=$this->data->getValue("soaexpire");
      if($this->data->getValue("ttl"))
       $content[6]=$this->data->getValue("ttl");
     }
     else
      $content=Array(
       $this->data->getValue("soaname"),
       str_replace("@", ".", $this->data->getValue("soaemail")),
       $this->data->getValue("soaserial"),
       $this->data->getValue("soarefresh"),
       $this->data->getValue("soaretry"),
       $this->data->getValue("soaexpire"),
       $this->data->getValue("ttl")
      );

     $this->data->setValue("content", implode(" ", $content));

     //convert IP to ARPA
     if(preg_match(REGEXP_DATA_IP, $this->data->getValue("name"))) //IPV4/IPV6 address
      if($arpa=$this->IP2ARPA($this->data->getValue("name"))) //try to convert IP address to ARPA address
       $this->data->setValue("name", $arpa);

     break;
    }
    default: break;
   }

   //store complete domain name
   if(($domain=$this->data->getValue("domain_id")))
   {
    $d=Data::cache("pdns_domains", "view", "item");
    $d->setShowFields("name");
    $d->setValue("id", $domain);

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    if(!$d->next())
    {
     $this->data->e->add("Unknown domain", "error");
     return false;
    }

    //correct entry name
    if(!($name=strtolower(trim($this->data->getValue("name"))))) //no record name
     $name=$d->v["name"]; //use domain name
    else if(preg_match("/^([a-z0-9\_\-]+)\.?$/", $name, $m)) //record
     $name=$m[1].".".$d->v["name"]; //append domain name
    else if(preg_match("/^([a-z0-9\_\-]+)\.".preg_quote($d->v["name"])."$/", $name, $m)) //record.domain.name
     $name=$m[1].".".$d->v["name"]; //append domain name
    else if($name!=$d->v["name"]) //record doesn't match domain
    {
     if(!preg_match("/\.(in-addr|ip6)\.arpa$/", $name)) //not an arpa domain
     {
      $this->data->e->add("Incorrect record name: \"".$name."\" for domain: \"".$d->v["name"]."\"", "error");
      return false;
     }
    }
    //else same as domain name

    $this->data->setValue("name", $name);
   }

   return true;
  }

  function addPostSubmitErrorCheck(){return $this->customizePostSubmitErrorCheck();}
  function copyPostSubmitErrorCheck(){return $this->customizePostSubmitErrorCheck();}
  function editPostSubmitErrorCheck(){return $this->customizePostSubmitErrorCheck();}

  function customizePostSubmitErrorCheck()
  {
   if($this->data->e->any("fatal")) //error check failed
    $this->data->setRequestValue("name"); //restore submitted domain name

   return true;
  }


  function addPreSubmit(){return $this->customizePreSubmit();}
  function copyPreSubmit(){return $this->customizePreSubmit();}
  function editPreSubmit(){return $this->customizePreSubmit();}

  function customizePreSubmit()
  {
   switch($this->data->getValue("type"))
   {
    case "A":
    case "AAAA": //add/update related pointer reference
    {
     if($this->data->getValue("ptr"))
     {
      $d=Data::cache("pdns_records", "view", "item");

      if($this->data->action=="edit")
      {
       $d->setShowFields("related_id");
       $d->setValue("id", $this->data->getValue("id"));

       if(!$d->get()||!$d->next())
       {
        $this->data->e->add($d->e);
        return false;
       }

       $ptr=$d->v["related_id"];
      }
      else //add/copy
       $ptr=false;

      if($ptr) //ptr should exist
      {
       //get ptr record
       $d->clearValues();
       $d->setValue("id", $ptr);

       if(!$d->get())
       {
        $this->data->e->add($d->e);
        return false;
       }

       if(!$d->next()) //ptr record doesn't exists
        $ptr=false; //create a new one
      }

      //add/update ptr record
      $d->clearValues();
      if($ptr) //ptr record exists
      {
       $d->setAction("edit");
       $d->setValue("id", $ptr);
      }
      else
       $d->setAction("add");

      $d->setValue("domain_id", $this->data->getValue("domain_id"));

      $d->setValue("type", "PTR");
      $d->setValue("name", $this->data->getValue("content"));
      $d->setValue("content", $this->data->getValue("name"));

      $d->setValue("ttl", $this->data->getValue("ttl"));
      $d->setValue("prio", $this->data->getValue("prio"));
      $d->setValue("change_date", $this->data->getValue("change_date"));
      $d->setValue("auth", $this->data->getValue("auth"));
      $d->setValue("disabled", $this->data->getValue("disabled"));
      $d->setValue("user", $this->data->getValue("user"));

      if($this->data->action=="edit") //should also be done at add....
       $d->setValue("related_id", $this->data->getValue("id"));

      if(!$d->submit())
      {
       $this->data->e->add($d->e);
       return false;
      }

      if(!$ptr) //get ptr record id
       $ptr=$d->getValue("id");

      $this->data->setValue("related_id", $ptr);
     }
     else //don't manage pointer record
     {
      //could clean up ptr record...

      $this->data->setValue("related_id", 0);
     }

     break;
    }
    case "MX":
    case "NS": //update corresponding A/AAAA/SOA records for NS/MX records
    {
     if($this->data->action!="edit") //not for add/copy
      break;

     //get old record values
     $d=Data::cache("pdns_records", "view", "item");
     $d->setShowFields("domain_id", "type", "name", "content", "dynamic");
     $d->clearValues();
     $d->setValue("id", $this->data->getValue("id"));

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     if(!$d->next()||$d->v["type"]!=$this->data->getValue("type")) //record not found or type didn't change
      break;

     if($d->v["content"]==$this->data->getValue("content")) //NS/MX record name didn't change
      break;

     $oldname=$d->v["content"];

     //get domain name
     $dd=Data::cache("pdns_domains", "view", "item");
     $dd->setShowFields("name");
     $dd->setValue("id", $d->v["domain_id"]);

     if(!$dd->get())
     {
      $this->data->e->add($dd->e);
      return false;
     }

     if(!$dd->next())
     {
      $this->data->e->add("Unknown domain", "error");
      return false;
     }

     if($oldname!=$dd->v["name"]) //name the same as domain name
     {
      //get corresponding A/AAAA/SOA records
      $d->setExtra("all");

      $d->clearValues();
      $d->setValue("type", "A", 0);
      $d->setValue("name", $oldname, 0);
      $d->setOperant("name", "OR", 0);
      $d->setValue("type", "AAAA", 1);
      $d->setValue("name", $oldname, 1);
      $d->setOperant("name", "OR", 1);

      if($this->data->getValue("type")=="NS")
      {
       $d->setValue("type", "SOA", 2);
       $d->setValue("content", $oldname, 2);
       $d->setOperator("content", "start like", 2);
      }

      if(!$d->get())
      {
       $this->data->e->add($d->e);
       return false;
      }

      $recordTypes=Array();
      $updates=Array();
      while($d->next())
      {
       if(!in_array($d->v["type"], $recordTypes))
        $recordTypes[]=$d->v["type"];

       if($d->v["type"]=="SOA")
       {
        $updates[]=Array(
         "id" => $d->v["id"],
         "domain_id" => $d->v["domain_id"],
         "type" => $d->v["type"],
         "name" => $d->v["name"],
         "content" => preg_replace("/^".preg_quote($oldname)." /", $this->data->getValue("content")." ", $d->v["content"]),
         "dynamic" => $d->v["dynamic"],
        );
       }
       else //A/AAAA
       {
        $updates[]=Array(
         "id" => $d->v["id"],
         "domain_id" => $d->v["domain_id"],
         "type" => $d->v["type"],
         "name" => $this->data->getValue("content"),
         "content" => $d->v["content"],
         "dynamic" => $d->v["dynamic"],
        );
       }
      }

      //update corresponding records
      if($updates)
      {
       $d->setAction("edit");
       $d->setExtra("item");

       foreach($updates as $c => $v)
       {
        $d->clearValues();

        $d->setValue("id", $v["id"], $c);
        $d->setValue("domain_id", $v["domain_id"], $c);
        $d->setValue("type", $v["type"], $c);
        $d->setValue("name", $v["name"], $c);
        $d->setValue("content", $v["content"], $c);
        $d->setValue("dynamic", $v["dynamic"], $c);

        if(!$d->moduleObject->customizeFields())
         return false;

        if(!$d->submit())
        {
         $this->data->e->add($d->e);
         return false;
        }
       }

       $this->data->e->add("Corresponding ".implode("/", $recordTypes)." records updated", "note");
      }
     }

     break;
    }
    default: break;
   }

   return true;
  }


  function addPostSubmit(){return $this->customizePostSubmit();}
  function copyPostSubmit(){return $this->customizePostSubmit();}
  function editPostSubmit(){return $this->customizePostSubmit();}

  function customizePostSubmit()
  {
   $this->data->setRequestValue("name"); //restore submitted domain name

   return true;
  }


  function getUserInfo()
  {
   //get all users
   $d=Data::cache("pdns_records", "view", "all");
   $d->setShowFields("user");
   $d->sortOrder("user");

   $d->setValue("aggregation_field", "user");
   $d->setValue("user", "");
   $d->setOperator("user", "!=");

   if($d->get()&&$d->vmax)
   {
    $options=Array("");

    while($d->next())
     $options[]=$d->v["user"];

    $this->data->listOptions("user", $options);
   }

   //get last used user
   if($this->data->action=="add"&&$this->data->check("valuetype", "default", "user"))
   {
    $d->sortOrder("change_date-");

    $d->setOperator("user", "=");
    $d->clearValues();
    $d->setValue("show_total", 1);

    if($d->get()&&$d->next())
     $this->data->setValue("user", $d->v["user"]);
   }

   return true;
  }

  function getRecords($domain)
  {
   $records=Array();

   $d=Data::cache("pdns_records", "view", "all");
   $d->setShowFields("type", "name", "content");

   $d->clearValues();
   $d->setValue("domain_id", $domain);

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return $records;
   }

   while($d->next())
   {
    if($d->v["type"]=="NS")
     $name=$d->v["content"];
    else
     $name=$d->v["name"];

    if(preg_match("/^([0-9\.]+|[0-F\:]+|[0-F\.]+\.(in-addr|ip6)\.arpa)$/i", $name)) //ignore IPs/ARPA
     continue;

    if(isset($records[$name]))
     $records[$name].=", ".$d->v["type"];
    else
     $records[$name]=$name." (".$d->v["type"];
   }

   if($records)
   {
    foreach(array_keys($records) as $name)
     $records[$name].=")";

    $records=array_merge(Array("" => ""), $records);
   }

   ksort($records);
   return $records;
  }

  function getIPs($domain, $ipv=false)
  {
   $ips=Array();

   $d=Data::cache("pdns_records", "view", "all");
   $d->setShowFields("domain_id", "type", "name", "content");
   $d->getReferenceData("domain_id")->setShowFields("name");

   $d->clearValues();
   $d->setValue("domain_id", $domain);

   if($ipv==6)
    $d->setValue("type", "AAAA");
   else if($ipv==4)
    $d->setValue("type", "A");
   else
   {
    $d->field["type"]["type"]="array";
    $d->setValue("type", Array("A", "AAAA"));
   }

   if(!$d->get(1))
   {
    $this->data->e->add($d->e);
    return $ips;
   }

   while($d->next())
   {
    if(!($ip=$d->v["content"]))
     continue;

    if($d->v["domain_id"]["name"]==$d->v["name"])
     $name=$d->v["name"];
    else
     $name=str_replace(".".$d->v["domain_id"]["name"], "", $d->v["name"]);

    if(isset($ips[$ip]))
     $ips[$ip].=", ".$name;
    else
     $ips[$ip]=($d->v["type"]=="A"?" ":"").$ip." (".$name; //add space to ensure A sorted on top
   }

   if($ips)
   {
    foreach(array_keys($ips) as $ip)
     $ips[$ip].=")";

    $ips=array_merge(Array("" => ""), $ips);
   }

   asort($ips); //sorts lowest ip first (unlike mysql)
   $ips=preg_replace("/^\s+/", "", $ips); //remove space prefix in A records

   return $ips;
  }

  function getDynamicIPKeys($domain)
  {
   $d=Data::cache("pdns_records", "view", "all");
   $d->setShowFields("dynamic");
   $d->sortOrder("dynamic+");

   $d->clearValues();
   $d->setValue("domain_id", $domain);

   $d->setValue("aggregation_field", "dynamic");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return Array();
   }

   return $d->field["dynamic"]["value"];
  }

  static function IP2ARPA($ip)
  {
   if(preg_match("/^[0-9\.]+$/", $ip)) //ipv4
   {
    $ip=explode(".", $ip);
    $arpa=implode(".", array_reverse($ip)).".in-addr.arpa"; //reverse and add suffix
   }
   else if(preg_match("/^[0-F\:]+$/i", $ip)) //ipv6
   {
    if(strstr($ip, "::")) //short version
     if(($sections=substr_count($ip, ":"))<7) //not enough sections
      $ip=str_replace("::", str_repeat(":", (7-$sections)+2), $ip); //replace short version with correct amount of sections

    $tmp="";
    $mutilate=new Mutilate();

    $ip=explode(":", $ip); //add leading zeros
    foreach($ip as $k => $v)
     $tmp.=$mutilate->leadingZeros($v, 4);

    $ip=Array(); //split in individual numbers
    $l=strlen($tmp);
    for($c=0;$c<$l;$c++)
     $ip[]=$tmp[$c];

    $arpa=implode(".", array_reverse($ip)).".ip6.arpa"; //reverse and add suffix
   }
   else
    return false;

   return $arpa;
  }

  static function ARPA2IP($arpa)
  {
   if(preg_match("/^([0-9\.]+)\.in-addr\.arpa$/", $arpa, $m)) //ipv4
   {
    $ip=explode(".", $m[1]);
    $ip=implode(".", array_reverse($ip)); //reverse address
   }
   else if(preg_match("/^([0-F\.]+)\.ip6\.arpa$/i", $arpa, $m)) //ipv6
   {
    $ip=explode(".", $m[1]);
    $ip=array_reverse($ip); //reverse address

    $l=sizeof($ip); //make sections of 4 numbers
    for($c=0;$c<$l;$c++)
    {
     if(!isset($tmp[floor($c/4)]))
      $tmp[floor($c/4)]="";

     $tmp[floor($c/4)].=$ip[$c];
    }

    $ip=""; //remove leading zeros and add :
    foreach($tmp as $section)
     $ip.=ltrim($section, "0").":";

    $ip=preg_replace("/\:{3,}/", "::", substr($ip, 0, -1)); //make short version and remove last :
   }
   else
    return false;

   return $ip;
  }

 }
?>
